import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.sql.SQLException;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import com.ephesoft.dcma.script.IJDomScript;

/**
 * The <code>ScriptEtractionr</code> class represents the script execute
 * structure. Writer of scripts plug-in should implement this IScript interface
 * to execute it from the scripting plug-in. Via implementing this interface
 * writer can change its java file at run time. Before the actual call of the
 * java Scripting plug-in will compile the java and run the new class file.
 * 
 * 
 * @version 1.0
 */


public class ScriptExtraction implements IJDomScript
{
	private static final String BATCH_LOCAL_PATH = "BatchLocalPath";
	private static final String BATCH_INSTANCE_ID = "BatchInstanceIdentifier";
	private static final String EXT_BATCH_XML_FILE = "_batch.xml";
	private static final String ZIP_FILE_EXT = ".zip";

	@Override
	public Object execute(Document document, String methodName, String docIdentifier)
	{
		Exception exception = null;
		try
		{
			System.out.println("*************  Inside ScriptExtraction scripts.");
			System.out.println("*************  Start execution of the ScriptExtraction scripts.");

			if (null == document)
			{
				System.out.println("Input document is null.");
				return exception;
			}
			documentTypeDecision(document);


			System.out.println("*************  End execution of the ScriptExtraction scripts.");
		}
		catch (Exception e)
		{
			System.out.println("*************  Error occurred in scripts." + e.getMessage());
			e.printStackTrace();
			exception = e;
		}
		return exception;
	}

	//Auto Learning Extraction

	private void documentTypeDecision(Document document) throws SQLException, IOException, JDOMException
	{
		//Gets the document root element.
		Element docRoot = document.getRootElement();


		//String variables containing information regarding the current batch class excuting this script.
		String batchClassId = docRoot.getChildText("BatchClassIdentifier");
		String batchLocalPath = docRoot.getChildText("BatchLocalPath");	
		docRoot.getChildText("BatchInstanceIdentifier");

		boolean POisPresent = true;

		//Get and traverse through documents list.
		List<Element> docList = docRoot.getChild("Documents").getChildren("Document");
		for (Element doc : docList)
		{
			doc.getChildText("Identifier");
			String docType = doc.getChildText("Type");	

			//Check if doc is Invoice
			if (docType.equals("INVOICE"))
			{
				//Get and traverse through document level fields list.
				List<Element> dlfList = doc.getChild("DocumentLevelFields").getChildren("DocumentLevelField");
				for (Element dlf: dlfList)
				{
					//String variables containing the name of the document level field, and the value in that field.
					//Note: If there is an empty tag in the batch xml, such as <Value/>, the text for that tag will be an empty string "", not null. 
					String dlfName = dlf.getChildText("Name");			

					String dlfValue = dlf.getChildText("Value");			
					if (dlfName.equals("PO_Number")){
						if(dlfValue!= null){
							POisPresent = false;
						}					
					}
					if(dlfName.equals("Line Items Matched") && !POisPresent){
						Element valueTag = new Element("Value");
						valueTag.setText("true");
						dlf.addContent(valueTag);
					}
				}
			}
		}
	}	
}
